/*
 * Copyright (c) 1997	XMaZ Project.
 *
 *	isoternet Protocol Library / iPv4 Module / output routine
 *
 *	$Id: ip_output.c,v 1.1 1997/09/10 15:43:25 nakamura Exp nakamura $
 */

#include <string.h>

#include <ttypes.h>
#include <inet.h>
#include <if.h>
#include "ip.h"

#define IPVHL	 0x45	/* Ver = 4, Header_Len = 5 DWRD's */
#define TTL	64	/* RFC 1340 */ 

void ip_output(BYTE FAR *p, WORD len)
{
	const routeentrytype **rtpp = 0;
	IP FAR *h;
	IF	*ifp;
	const routeentrytype *rtp;

	/* header generation */

	h = (IP FAR *) p;

	h->ip_vhl = IPVHL;
	h->ip_tos = 0;
	h->ip_id = inet_word(id++);
	h->ip_flr = 0;
/*	h->ip_ttl = TTL; */
/*	h->ip_proto = proto; */
/*	h->ip_dst = inet_dword(dst); */

	/* dst interface selection */

	if (rtpp == 0) {
		rtp=ip_rtsearch(h->ip_dst);
	}
	else rtp=*rtpp;

	if (rtp == 0) {
#ifdef DEBUG
	log_printf("iP_output: No route.");
#endif
		tmem_free(p);
		return;
	}

	ifp = rtp->ifp;
	h->ip_src = ifp->if_addr;	/* routing is not implemented */

	/* output */

#ifdef FRAGMENTATION

	while (ifp->if_mtu < (IPHLEN + len)) {
		WORD datalen;

		h->ip_flr |= 0x0020;
		datalen = (ifp->if_mtu - IPHLEN) & 0xfff8;
		h->ip_len = inet_dword(IPHLEN + datalen);
		h->ip_cksum = 0;
		h->ip_cksum = in_cksum(h, IPHLEN);
		ifp->if_output(p, IPHLEN + datalen);

		h->ip_flr = inet_word(inet_word(h->ip_flr) + datalen);
		len -= datalen;
		
		memcpy((BYTE FAR *) h, (BYTE FAR *) h + datalen, IPHLEN);
	}

	h->ip_flr &= ~ 0x0020;	/* IP forward is not supported */

#else

	if (ifp->if_mtu < (IPHLEN + len)) {
#ifdef DEBUG
	log_printf("iP_output: Too large packet.");
#endif
		tmem_free(p);
		return;
	}

#endif /* FRAGMENTATION */

	h->ip_len = inet_word(IPHLEN + len);
	h->ip_cksum = 0;
	h->ip_cksum = in_cksum((BYTE FAR *)h, IPHLEN);
#ifdef DEBUG
	log_printf("iP_output:");
	log_putbinary(p,IPHLEN+len);
#endif
	ifp->if_output(p, IPHLEN + len, 
							(rtp->flag.gateway) ? rtp->gateway : h->ip_dst);

	return;
} 


/*
     :
1998/ 9/16
2000/ 9/23 ip_rtsearchgpBif_output ňĐw
2001/12/ 9 pPbgŜOɏo
*/
